#!/usr/bin/env python
#
# Freeciv - Copyright (C) 2003 - The Freeciv Project
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2, or (at your option)
#   any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
# lists.def
#   <name of output file without extension>
#   <list name>\t<struct name>
#   <list name>\t<struct name>
#   <list name>\t<struct name>
#   ...

import re, string, os, sys

def write_disclaimer(f):
    f.write('''/****************************************************************************
 *                       THIS FILE WAS GENERATED                             *
 * Script: genlist.py                                                        *
 * Input:  lists.def                                                         *
 *                       DO NOT CHANGE THIS FILE                             *
 ****************************************************************************/
''')

def verbose(s):
    if len(sys.argv) > 1 and sys.argv[1] == "-v":
        print s

def my_open(name):
    verbose("writing %s" % name)
    f=open(name, "w")
    write_disclaimer(f)
    return f

def main():
    content = open("lists.def").read()
    lines = string.split(content, "\n")
    lines = map(lambda x: re.sub("#.*$","",x), lines)
    lines = map(lambda x: re.sub("//.*$","",x), lines)
    lines = filter(lambda x:not re.search("^\s*$",x), lines)
    outfile = lines[0]
    del lines[0]
    f = my_open('%s.h' % outfile)
    f.write('#ifndef FC__%s_H\n' % outfile.upper())
    f.write('#define FC__%s_H\n\n' % outfile.upper())
    for l in lines:
      listname, structname = string.split(l, "\t")
      f.write('#include "genlist.h"\n\n')
      f.write('struct %s {\n' % listname)
      f.write('  struct genlist list;\n')
      f.write('};\n\n')

      def funcit(r, u, e, body):
        f.write('static inline %s %s_%s(struct %s *tthis%s)\n' 
                 % (u, listname, r, listname, e))
        f.write('{\n %s \n}\n\n' % body)

      funcit('init', 'void', '', '  genlist_init(&tthis->list);');
      funcit('size', 'int', '', '  return genlist_size(&tthis->list);');
      funcit('get', 'struct %s *' % structname, ', int index',
             '  return (struct %s *)genlist_get(&tthis->list, index);' % structname)
      funcit('insert', 'void', ', struct %s *pfoo' % structname, 
             '  genlist_insert(&tthis->list, pfoo, 0);')
      funcit('insert_back', 'void', ', struct %s *pfoo' % structname,
             '  genlist_insert(&tthis->list, pfoo, -1);')
      funcit('unlink', 'void', ', struct %s *pfoo' % structname,
             '  genlist_unlink(&tthis->list, pfoo);')
      funcit('unlink_all', 'void', '', '  genlist_unlink_all(&tthis->list);')
      funcit('sort', 'void', 
             ',\n\t int (*compar)(const void *, const void *)',
             '  genlist_sort(&tthis->list, compar);')

      f.write('#define %s_iterate(list, plist) \\\n' % listname)
      f.write('TYPED_LIST_ITERATE(struct %s, list, plist)\n' % structname)
      f.write('#define %s_iterate_end  LIST_ITERATE_END\n' % listname)
      f.write('#define %s_iterate_rev(list, plist) \\\n' % listname)
      f.write('TYPED_LIST_ITERATE_REV(struct %s, list, plist)\n' % structname)
      f.write('#define %s_iterate_rev_end  LIST_ITERATE_END\n' % listname)

    f.write('\n#endif\n')

    f.close()

main()
